﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using PaxScript.Net;

namespace DebugApp
{
    public partial class Form1 : Form
    {
        private PaxScript.Net.PaxScripter paxScripter1;

        public Form1()
        {
            InitializeComponent();
            paxScripter1 = new PaxScript.Net.PaxScripter(this.components);
            paxScripter1.DebugMode = true;
            this.paxScripter1.State = PaxScript.Net.ScripterState.Init;
            this.paxScripter1.OnChangeState += new PaxScript.Net.ChangeStateHandler(this.paxScripter1_OnChangeState);
            this.paxScripter1.OnPrint += DoPrint;
        }

        private void DoPrint(PaxScripter sender, string s)
        {
            tbState.Text += "\r\n" + "Output :" + s;
        }
        private void btnReset_Click(object sender, EventArgs e)
        {
            paxScripter1.Reset();
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            paxScripter1.Run(RunMode.Run);
        }

        private void btnTraceInto_Click(object sender, EventArgs e)
        {
            paxScripter1.Run(RunMode.TraceInto);
        }

        private void btnStepOver_Click(object sender, EventArgs e)
        {
            paxScripter1.Run(RunMode.StepOver);
        }

        private void btnAddBreakpoint_Click(object sender, EventArgs e)
        {
            paxScripter1.AddBreakpoint("1", 5);
            ShowInfo(paxScripter1, paxScripter1.State);
        }

        private void btnRemoveBreakpoint_Click(object sender, EventArgs e)
        {
            paxScripter1.RemoveAllBreakpoints();
            ShowInfo(paxScripter1, paxScripter1.State);
        }

        private void ShowInfo(PaxScript.Net.PaxScripter sender, ScripterState new_state)
        {
            if (tbState.Text == "")
              tbState.Text = "State: " + new_state.ToString();
            else
              tbState.Text += "\r\n" + "Module: " + sender.CurrentModule;
            object v = null;

            if (new_state == ScripterState.Paused)
            {
                tbState.Text += "\r\n" + "Line " + sender.CurrentLineNumber.ToString() +
                    " : " + sender.CurrentLine;
                v = sender.Eval("number");
                sender.DiscardError();
            }

            if (v == null)
                tbState.Text += "\r\n" + "number = null";
            else
                tbState.Text += "\r\n" + "number = " + v.ToString();

            tbState.Text += "\r\n" + "Call stack:";
            foreach (CallStackRec csr in sender.Call_Stack)
            {
                tbState.Text += "\r\n" + csr.CallView + " at line " + csr.LineNumber;
            }

            tbState.Text += "\r\n" + "Breakpoints:";
            foreach (Breakpoint bp in sender.Breakpoint_List)
            {
                tbState.Text += "\r\n" + "Breakpoint at line " + bp.LineNumber +
                    ", Activated = " + bp.Activated;
            }

            tbState.SelectionStart = tbState.Text.Length;
            tbState.ScrollToCaret();
        }
        private void paxScripter1_OnChangeState(PaxScript.Net.PaxScripter sender, PaxScript.Net.ChangeStateEventArgs e)
        {
            if (!sender.HasErrors && sender.State == ScripterState.Running)
                ShowInfo(sender, e.NewState);
            if (e.NewState == ScripterState.Error)
            {
                MessageBox.Show("Error: " + sender.Error_List[0].Message + "\r\n" +
                    "Line number:" + sender.Error_List[0].LineNumber.ToString());
            }
            else if (e.OldState == ScripterState.Init)
            {
                sender.AddModule("1");
                sender.AddCode("1", tbScript.Text);
            }
        }
    }
}